@@ -1,6 +1,6 @@
 {-# LANGUAGE CPP #-}
 -- -----------------------------------------------------------------------------
--- 
+--
 -- Main.hs, part of Alex
 --
 -- (c) Chris Dornan 1995-2000, Simon Marlow 2003
@@ -67,7 +67,7 @@ alexOpenFile file mode = do
 alexOpenFile = openFile
 #endif
 
--- `main' decodes the command line arguments and calls `alex'.  
+-- `main' decodes the command line arguments and calls `alex'.
 
 main:: IO ()
 main =  do
@@ -78,7 +78,7 @@ main =  do
         bye (usageInfo (usageHeader prog) argInfo)
     (cli,_,[]) | DumpVersion `elem` cli ->
         bye copyright
-    (cli,[file],[]) -> 
+    (cli,[file],[]) ->
         runAlex cli file
     (_,_,errors) -> do
         prog <- getProgramName
@@ -98,7 +98,7 @@ runAlex cli file = do
   basename <- case (reverse file) of
                 'x':'.':r -> return (reverse r)
                 _         -> die (file ++ ": filename must end in \'.x\'\n")
-  
+
   prg <- alexReadFile file
   script <- parseScript file prg
   alex cli file basename script
@@ -107,7 +107,7 @@ parseScript :: FilePath -> String
   -> IO (Maybe (AlexPosn,Code), [Directive], Scanner, Maybe (AlexPosn,Code))
 parseScript file prg =
   case runP prg initialParserEnv parse of
-        Left (Just (AlexPn _ line col),err) -> 
+        Left (Just (AlexPn _ line col),err) ->
                 die (file ++ ":" ++ show line ++ ":" ++ show col
                                  ++ ": " ++ err ++ "\n")
         Left (Nothing, err) ->
@@ -119,26 +119,26 @@ alex :: [CLIFlags] -> FilePath -> FilePath
      -> (Maybe (AlexPosn, Code), [Directive], Scanner, Maybe (AlexPosn, Code))
      -> IO ()
 alex cli file basename script = do
-   (put_info, finish_info) <- 
+   (put_info, finish_info) <-
       case [ f | OptInfoFile f <- cli ] of
            []  -> return (\_ -> return (), return ())
            [Nothing] -> infoStart file (basename ++ ".info")
            [Just f]  -> infoStart file f
            _   -> dieAlex "multiple -i/--info options"
-   
+
    o_file <- case [ f | OptOutputFile f <- cli ] of
                 []  -> return (basename ++ ".hs")
                 [f] -> return f
                 _   -> dieAlex "multiple -o/--outfile options"
-  
+
    tab_size <- case [ s | OptTabSize s <- cli ] of
-                []  -> return 8
+                []  -> return (8 :: Int)
                 [s] -> case reads s of
                         [(n,"")] -> return n
                         _        -> dieAlex "-s/--tab-size option is not a valid integer"
                 _   -> dieAlex "multiple -s/--tab-size options"
 
-   let target 
+   let target
         | OptGhcTarget `elem` cli = GhcTarget
         | otherwise               = HaskellTarget
 
@@ -147,18 +147,21 @@ alex cli file basename script = do
         | otherwise            = []
 
    template_dir  <- templateDir getDataDir cli
-                
+
+   let (maybe_header, directives, scanner1, maybe_footer) = script
+
+   scheme <- getScheme directives
+
    -- open the output file; remove it if we encounter an error
-   bracketOnError 
+   bracketOnError
         (alexOpenFile o_file WriteMode)
         (\h -> do hClose h; removeFile o_file)
         $ \out_h -> do
 
    let
-         (maybe_header, directives, scanner1, maybe_footer) = script
+         wrapper_name = wrapperFile template_dir scheme
          (scanner2, scs, sc_hdr) = encodeStartCodes scanner1
-         (scanner_final, actions) = extractActions scanner2
-
+         (scanner_final, actions) = extractActions scheme scanner2
          encodingsScript = [ e | EncodingDirective e <- directives ]
 
    encoding <- case nub (encodingsCli ++ encodingsScript) of
@@ -167,8 +170,6 @@ alex cli file basename script = do
      _ | null encodingsCli -> dieAlex "conflicting %encoding directives"
        | otherwise -> dieAlex "--latin1 flag conflicts with %encoding directive"
 
-   wrapper_name <- wrapperFile template_dir directives
-
    hPutStr out_h (optsToInject target cli)
    injectCode maybe_header file out_h
 
@@ -188,7 +189,7 @@ alex cli file basename script = do
        nm  = scannerName scanner_final
        usespreds = usesPreds min_dfa
 
-   
+
    put_info "\nStart codes\n"
    put_info (show $ scs)
    put_info "\nScanner\n"
@@ -199,7 +200,7 @@ alex cli file basename script = do
    put_info (infoDFA 1 nm dfa "")
    put_info "\nMinimized DFA"
    put_info (infoDFA 1 nm min_dfa "")
-   hPutStr out_h (outputDFA target 1 nm min_dfa "")
+   hPutStr out_h (outputDFA target 1 nm scheme min_dfa "")
 
    injectCode maybe_footer file out_h
 
@@ -214,6 +215,117 @@ alex cli file basename script = do
    hClose out_h
    finish_info
 
+getScheme :: [Directive] -> IO Scheme
+getScheme directives =
+  do
+    token <- case [ ty | TokenType ty <- directives ] of
+      [] -> return Nothing
+      [res] -> return (Just res)
+      _ -> dieAlex "multiple %token directives"
+
+    action <- case [ ty | ActionType ty <- directives ] of
+      [] -> return Nothing
+      [res] -> return (Just res)
+      _ -> dieAlex "multiple %action directives"
+
+    typeclass <- case [ tyclass | TypeClass tyclass <- directives ] of
+      [] -> return Nothing
+      [res] -> return (Just res)
+      _ -> dieAlex "multiple %typeclass directives"
+
+    case [ f | WrapperDirective f <- directives ] of
+        []  ->
+          case (typeclass, token, action) of
+            (Nothing, Nothing, Nothing) ->
+              return Default { defaultTypeInfo = Nothing }
+            (Nothing, Nothing, Just actionty) ->
+              return Default { defaultTypeInfo = Just (Nothing, actionty) }
+            (Just _, Nothing, Just actionty) ->
+              return Default { defaultTypeInfo = Just (typeclass, actionty) }
+            (_, Just _, _) ->
+              dieAlex "%token directive only allowed with a wrapper"
+            (Just _, Nothing, Nothing) ->
+              dieAlex "%typeclass directive without %token directive"
+        [single]
+          | single == "gscan" ->
+            case (typeclass, token, action) of
+              (Nothing, Nothing, Nothing) ->
+                return GScan { gscanTypeInfo = Nothing }
+              (Nothing, Just tokenty, Nothing) ->
+                return GScan { gscanTypeInfo = Just (Nothing, tokenty) }
+              (Just _, Just tokenty, Nothing) ->
+                return GScan { gscanTypeInfo = Just (typeclass, tokenty) }
+              (_, _, Just _) ->
+                dieAlex "%action directive not allowed with a wrapper"
+              (Just _, Nothing, Nothing) ->
+                dieAlex "%typeclass directive without %token directive"
+          | single == "basic" || single == "basic-bytestring" ||
+            single == "strict-bytestring" ->
+            let
+              strty = case single of
+                "basic" -> Str
+                "basic-bytestring" -> Lazy
+                "strict-bytestring" -> Strict
+                _ -> error "Impossible case"
+            in case (typeclass, token, action) of
+              (Nothing, Nothing, Nothing) ->
+                return Basic { basicStrType = strty,
+                               basicTypeInfo = Nothing }
+              (Nothing, Just tokenty, Nothing) ->
+                return Basic { basicStrType = strty,
+                               basicTypeInfo = Just (Nothing, tokenty) }
+              (Just _, Just tokenty, Nothing) ->
+                return Basic { basicStrType = strty,
+                               basicTypeInfo = Just (typeclass, tokenty) }
+              (_, _, Just _) ->
+                dieAlex "%action directive not allowed with a wrapper"
+              (Just _, Nothing, Nothing) ->
+                dieAlex "%typeclass directive without %token directive"
+          | single == "posn" || single == "posn-bytestring" ->
+            let
+              isByteString = single == "posn-bytestring"
+            in case (typeclass, token, action) of
+              (Nothing, Nothing, Nothing) ->
+                return Posn { posnByteString = isByteString,
+                              posnTypeInfo = Nothing }
+              (Nothing, Just tokenty, Nothing) ->
+                return Posn { posnByteString = isByteString,
+                              posnTypeInfo = Just (Nothing, tokenty) }
+              (Just _, Just tokenty, Nothing) ->
+                return Posn { posnByteString = isByteString,
+                              posnTypeInfo = Just (typeclass, tokenty) }
+              (_, _, Just _) ->
+                  dieAlex "%action directive not allowed with a wrapper"
+              (Just _, Nothing, Nothing) ->
+                dieAlex "%typeclass directive without %token directive"
+          | single == "monad" || single == "monad-bytestring" ||
+            single == "monadUserState" ||
+            single == "monadUserState-bytestring" ->
+            let
+              isByteString = single == "monad-bytestring" ||
+                             single == "monadUserState-bytestring"
+              userState = single == "monadUserState" ||
+                          single == "monadUserState-bytestring"
+            in case (typeclass, token, action) of
+              (Nothing, Nothing, Nothing) ->
+                return Monad { monadByteString = isByteString,
+                               monadUserState = userState,
+                               monadTypeInfo = Nothing }
+              (Nothing, Just tokenty, Nothing) ->
+                return Monad { monadByteString = isByteString,
+                               monadUserState = userState,
+                               monadTypeInfo = Just (Nothing, tokenty) }
+              (Just _, Just tokenty, Nothing) ->
+                return Monad { monadByteString = isByteString,
+                               monadUserState = userState,
+                               monadTypeInfo = Just (typeclass, tokenty) }
+              (_, _, Just _) ->
+                  dieAlex "%action directive not allowed with a wrapper"
+              (Just _, Nothing, Nothing) ->
+                dieAlex "%typeclass directive without %token directive"
+          | otherwise -> dieAlex ("unknown wrapper type " ++ single)
+        _many -> dieAlex "multiple %wrapper directives"
+
 -- inject some code, and add a {-# LINE #-} pragma at the top
 injectCode :: Maybe (AlexPosn,Code) -> FilePath -> Handle -> IO ()
 injectCode Nothing _ _ = return ()
@@ -280,7 +392,7 @@ templateDir def cli
 templateFile :: FilePath -> Target -> UsesPreds -> [CLIFlags] -> FilePath
 templateFile dir target usespreds cli
   = dir ++ "/AlexTemplate" ++ maybe_ghc ++ maybe_debug ++ maybe_nopred
-  where 
+  where
         maybe_ghc = case target of
                       GhcTarget -> "-ghc"
                       _         -> ""
@@ -295,12 +407,11 @@ templateFile dir target usespreds cli
                           && null maybe_debug -> "-nopred"
             _                                 -> ""
 
-wrapperFile :: FilePath -> [Directive] -> IO (Maybe FilePath)
-wrapperFile dir directives =
-  case [ f | WrapperDirective f <- directives ] of
-        []  -> return Nothing
-        [f] -> return (Just (dir ++ "/AlexWrapper-" ++ f))
-        _many -> dieAlex "multiple %wrapper directives"
+wrapperFile :: FilePath -> Scheme -> Maybe FilePath
+wrapperFile dir scheme =
+  do
+    f <- wrapperName scheme
+    return (dir ++ "/AlexWrapper-" ++ f)
 
 infoStart :: FilePath -> FilePath -> IO (String -> IO (), IO ())
 infoStart x_file info_file = do
@@ -314,7 +425,7 @@ infoStart x_file info_file = do
 infoHeader :: Handle -> FilePath -> IO ()
 infoHeader h file = do
 --  hSetBuffering h NoBuffering
-  hPutStrLn h ("Info file produced by Alex version " ++ projectVersion ++ 
+  hPutStrLn h ("Info file produced by Alex version " ++ projectVersion ++
                 ", from " ++ file)
   hPutStrLn h hline
   hPutStr h "\n"
@@ -325,7 +436,7 @@ initialParserEnv = (initSetEnv, initREEnv)
 initSetEnv :: Map String CharSet
 initSetEnv = Map.fromList [("white", charSet " \t\n\v\f\r"),
                            ("printable", charSetRange (chr 32) (chr 0x10FFFF)), -- FIXME: Look it up the unicode standard
-                           (".", charSetComplement emptyCharSet 
+                           (".", charSetComplement emptyCharSet
                             `charSetMinus` charSetSingleton '\n')]
 
 initREEnv :: Map String RExp
@@ -334,7 +445,7 @@ initREEnv = Map.empty
 -- -----------------------------------------------------------------------------
 -- Command-line flags
 
-data CLIFlags 
+data CLIFlags
   = OptDebugParser
   | OptGhcTarget
   | OptOutputFile FilePath
@@ -394,8 +505,8 @@ bracketOnError
         -> IO c         -- returns the value from the in-between computation
 bracketOnError before after thing =
   block (do
-    a <- before 
-    r <- Exception.catch 
+    a <- before
+    r <- Exception.catch
            (unblock (thing a))
            (\e -> do { after a; throw e })
     return r
